<?php
/**
 *
 * @category        admin
 * @package         users
 * @author          WebsiteBaker Project
 * @copyright       Ryan Djurovich
 * @copyright       WebsiteBaker Org. e.V.
 * @link            http://websitebaker.org/
 * @license         http://www.gnu.org/licenses/gpl.html
 * @platform        WebsiteBaker 2.10.1
 * @requirements    PHP 5.6 and higher
 * @version         $Id: save.php 250 2019-03-17 16:24:20Z Luisehahne $
 * @filesource      $HeadURL: svn://isteam.dynxs.de/wb/2.12.x/branches/main/admin/users/save.php $
 * @lastmodified    $Date: 2019-03-17 17:24:20 +0100 (So, 17. Mrz 2019) $
 *
 */

//  Print admin header
if (!\defined('SYSTEM_RUN')) {require( (\dirname(\dirname((__DIR__)))).'/config.php');}
//  suppress to print the header, so no new FTAN will be set
    $admin = new \admin('Access', 'users_modify', false);
    $oTrans = \Translate::getInstance();
    $oTrans->enableAddon(ADMIN_DIRECTORY.'\\users');

    $aInputs = [];
    $aErrorMessage = [];
// Create a javascript back link
    $sAddonBackUrl = ADMIN_URL.'/users/index.php';
try {

    if (!$admin->checkFTAN() )
    {
        $admin->print_header();
        $sInfo = \strtoupper(\basename(__DIR__).'_'.\basename(__FILE__, ''.PAGE_EXTENSION).'::');
        $sDEBUG=(\defined('DEBUG') && DEBUG ? $sInfo : '');
        throw new \Exception ($oTrans->MESSAGE_GENERIC_SECURITY_ACCESS);
    }

// After check print the header
    $admin->print_header();

    $aInputs = \array_merge( $_POST );
// Check if user id is a valid number and doesnt equal 1
    if (!isset($aInputs['user_id']) || !\is_numeric($aInputs['user_id']) || $aInputs['user_id'] == 1) {
        header("Location: index.php");
        exit(0);
    } else {
        $user_id = \intval($aInputs['user_id']);
    }

// Gather details entered
    $groups_id = ( isset($aInputs['groups']) ? \implode(",", $aInputs['groups']) : '');
    $active = \intval( \is_array($aInputs['active'])  ?($aInputs['active'][0]):$aInputs['active']);

    $password     = \preg_replace('/[^\x20-\x7E]+$]/', '',$admin->StripCodeFromText($admin->get_post('password')));
    $password2    = \preg_replace('/[^\x20-\x7E]+$]/', '',$admin->StripCodeFromText($admin->get_post('password2')));
    $display_name = $admin->StripCodeFromText(($admin->get_post('display_name')));
    $email        = $admin->StripCodeFromText($admin->get_post('email'));
    $home_folder  = $admin->StripCodeFromText($admin->get_post('home_folder'));

// Check values
    if($groups_id == "") {
        $aErrorMessage[] = ($oTrans->MESSAGE_USERS_NO_GROUP);
        throw new \Exception ($oTrans->MESSAGE_USERS_NO_GROUP);
    }

    if ($password != "") {
        if (strlen($password) < 6) {
            $aErrorMessage[] = ($oTrans->MESSAGE_USERS_PASSWORD_TOO_SHORT);
            throw new \Exception ($oTrans->MESSAGE_USERS_PASSWORD_TOO_SHORT);
        }
        if ($password != $password2) {
            $aErrorMessage[] = ($oTrans->MESSAGE_USERS_PASSWORD_MISMATCH);
            throw new \Exception ($oTrans->MESSAGE_USERS_PASSWORD_MISMATCH);
        }
    }
    $md5_password =  md5($password);

    if($email != "")
    {
        if($admin->validate_email($email) == false)
        {
            $aErrorMessage[] = ($oTrans->MESSAGE_USERS_INVALID_EMAIL);
            throw new \Exception ($oTrans->MESSAGE_USERS_INVALID_EMAIL);
        }
    } else { // e-mail must be present
        $aErrorMessage[] = ($oTrans->MESSAGE_SIGNUP_NO_EMAIL);
        throw new \Exception ($oTrans->MESSAGE_SIGNUP_NO_EMAIL);
    }

// Check if the email already exists
    $sql  = 'SELECT `user_id` FROM `'.TABLE_PREFIX.'users` '
          . 'WHERE `email` = \''.$email.'\' '
          .   'AND `user_id` <> '.$user_id;
    if ($database->get_one($sql))
    {
        if(isset($oTrans->MESSAGE_USERS_EMAIL_TAKEN))
        {
            $aErrorMessage[] = ($oTrans->MESSAGE_USERS_EMAIL_TAKEN);
            throw new \Exception (\sprintf($oTrans->MESSAGE_USERS_EMAIL_TAKEN));
        }
    }
    $sql  = 'SELECT COUNT(*) FROM `'.TABLE_PREFIX.'users` ';
    $sql .= 'WHERE `user_id` <> '.$user_id.' AND `display_name` LIKE \''.$display_name.'\'';
    if ($numRow=$database->get_one($sql)) {
        $sErrorMessage = $aErrorMessage[] = ( @$oTrans->MESSAGE_USERS_DISPLAYNAME_TAKEN ? : $oTrans->MESSAGE_MEDIA_BLANK_NAME.' ('.$oTrans->TEXT_DISPLAY_NAME.')');
        throw new \Exception ($sErrorMessage);
    }

    if (!sizeof($aErrorMessage)) {
    // Update the database
    $sql  = 'UPDATE `'.TABLE_PREFIX.'users` SET '
          . '`groups_id` = \''.$database->escapeString($groups_id).'\', '
          . '`active` = '.$database->escapeString($active).', '
          . '`display_name` = \''.$database->escapeString($display_name).'\', '
          . '`home_folder` = \''.$database->escapeString($home_folder).'\', '
          . '`email` = \''.$database->escapeString($email).'\''
          . ( ($password == "") ? ' ': ', `password` = \''.$database->escapeString($md5_password).'\' ' )
          . 'WHERE `user_id` = '.$database->escapeString($user_id);
        if (!$database->query($sql)) {
            if($database->is_error()) {
                $sErrorMessage = $aErrorMessage[] = $database->get_error();
                throw new \Exception ($sErrorMessage);
            }
        }
    }

    if (isset($_SESSION['users'])) {unset($_SESSION['users']);}
    $admin->print_success($oTrans->MESSAGE_USERS_SAVED, $sAddonBackUrl);

} catch (\Exception $ex) {
    if (!function_exists ('xnl2br')) {require(WB_PATH . '/framework/functions.php');}
    $sErrMsg = xnl2br(\sprintf('[%d] %s', $ex->getLine(), $ex->getMessage()));
    $admin->print_error ($sErrMsg, $sAddonBackUrl);
    exit;
}
// Print admin footer
$admin->print_footer();
