<?php
/**
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Description of Lingual
 *
 * @package      Addon package
 * @copyright    Dietmar Wöllbrink
 * @author       Dietmar Wöllbrink
 * @author       Manuela v.d.Decken
 * @license      GNU General Public License 2.0
 * @version      1.0.0-dev.0
 * @revision     $Id: Lingual.php 234 2019-03-17 06:05:56Z Luisehahne $
 * @since        File available since 02.12.2017
 * @deprecated   no
 * @description  xxx
 *
 */
//declare(strict_types = 1);
//declare(encoding = 'UTF-8');

namespace addon\WBLingual;

use bin\WbAdaptor;

/* -------------------------------------------------------- */
/* --------------------------------------------------------
// Must include code to prevent this file from being accessed directly
if (!\defined('SYSTEM_RUN')) {\header($_SERVER['SERVER_PROTOCOL'].' 404 Not Found'); echo '404 Not Found'; \flush(); exit;}
    if (\version_compare(WB_VERSION, '2.11.0', '<')){
        throw new \Exception ('It is not possible to upgrade from WebsiteBaker Versions before 2.11.0');
    }
*/
class Lingual {

/** holds the active singleton instance */
    private static $oInstance = null;
/** @var object instance of the WbAdaptor object */
    protected $oReg   = null;
/** @var object instance of the application object */
    private $oApp     = null;
/** @var object instance of the database object */
    private $oDb      = null;

/** @var array holds several default values  */
    private $aConfig     = [];
/** @var array set several values for Twig_Environment */
    private $aTwigEnv    = [];
/** @var array set several values for Twig_Loader */
    private $aTwigLoader = [];
/** @var string set icon extension */
    private $sExtension  = 'png';

/**
 * constructor used to import some application constants and objects
 */
    public function __construct(){
        $this->init();
    }

    private function init(){
        $oReg        = WbAdaptor::getInstance();
        $this->oReg  = $oReg;
        $this->oApp  = $oReg->App;
        $this->oDb   = $oReg->Db;
        $this->aConfig['aLang'] = $this->getLanguageInUse();
        $this->aConfig['Twig']  = $this->getTwigConfig();
        $this->aTwigLoader      = $this->Twig['twig-loader-file'];
        $this->aTwigEnv         = $this->Twig['twig-environment'];
        $this->getTemplatePath();
    }

    /**
     * CopyAddons::__set()
     *
     * @param mixed $name
     * @param mixed $value
     * @return
     */
    public function __set($name, $value)
    {
       return $this->config[$name] = $value;
    }

    /**
     * CopyAddons::__isset()
     *
     * @param mixed $name
     * @return
     */
    public function __isset($name)
    {
        return isset($this->aConfig[$name]);
    }

    public function __get($name)
    {
        if (!$this->__isset($name)) {
            throw new \Exception('Tried to get none existing property ['.__CLASS__.'::'.$name.']');
        }
        return $this->aConfig[$name];
    }

/********************************************************************************************/
//
/********************************************************************************************/

    public function set($name, $value = '')
    {
        $this->aConfig[$name] = $value;
    }

    public function get($name)
    {
        return $this->$name;
    }
/********************************************************************************************/
//
/********************************************************************************************/

/**
 * get a valid instance of this class
 * @return object
 */
    static public function getLingualRel() {
      $sLingualDir = \str_replace('\\','/',\str_replace(WB_PATH,'',__DIR__.'/update_keys.php'));
      return $sLingualDir;
    }

    /**
    * Lib::getLangMenu()
    *
    * @param mixed $config
    * @param mixed $oApp
    * @return
    */
    private function getLangMenuTwig ( )
    {
        $sRetval = '';
        $data['aTargetList'] = $this->getLangMenuData();
        if (\sizeof($data['aTargetList'])>1){
            $loader  = new \Twig_Loader_Filesystem($this->AbsTemplateDir.$this->aTwigLoader['templates_dir']);
            $twig    = new \Twig_Environment($loader);
            $sRetval = $twig->render($this->aTwigLoader['default_template'], $data);
        }
        return $sRetval;
    }

    private function getTemplatePath()
    {
        $this->aConfig['AbsTemplateDir'] = $this->oReg->TemplatePath;
        if (is_readable($this->AbsTemplateDir.$this->aTwigLoader['templates_dir'].'/'.$this->aTwigLoader['default_template'])){
        /* do nothing */
        } else if (is_readable(str_replace('\\','/',__DIR__).'/'.$this->aTwigLoader['templates_dir'].'/')){
            $this->aConfig['AbsTemplateDir'] = str_replace('\\','/',__DIR__).'/';
        }
    }

    private function getTwigConfig(){
        $sFilename = __DIR__.'/default.ini';
        return (\is_readable($sFilename) ? $this->getTwigEnv($sFilename) : false );
    }

    private function getTwigEnv($sFilename)
    {
        $aRetval = \parse_ini_file($sFilename, true, INI_SCANNER_TYPED);
        return $aRetval;
    }

/********************************************************************************************/
//
/********************************************************************************************/

    /**
     * methode to update a var/value-pair into table
     * @param integer $iPageId which page shall be updated
     * @param string $sTable the pages table
     * @param integer $iEntry
     * @return bool
     */
    private function updatePageCode($iPageId, $sTable, $iNewPageCode = null)
    {
        // if new Pagecode is missing then set the own page ID
        $entry = ( !isset($iNewPageCode) ? $iPageId : $iNewPageCode);
        $sql = 'UPDATE `'.$this->oDb->sTablePrefix.$sTable.'` '
             . 'SET `page_code`='.$entry.', '
             .     '`modified_when` = '.\time().' '
             . 'WHERE `page_id` = '.$iPageId;
        return (bool)$this->oDb->query($sql);
    }

    private function getLanguageInUse($sLangKey='') {
      $aResult = [];
      $sqlSet = 'SELECT  DISTINCT '
             . '`language`,`page_id`,`level`,`parent`,`root_parent`,`page_code`,`link`'
             . ',`visibility`,`viewing_groups`,`viewing_users`,`position`'
             . ',`page_title`,`tooltip` '
              . 'FROM `'.$this->oDb->sTablePrefix.'pages` '
              . 'WHERE `level`= 0 '
//              .   'AND `visibility` NOT IN(\'none\', \'deleted\') '
              .   'AND `visibility` NOT IN(\'none\', \'hidden\', \'deleted\') '
              .   (($sLangKey!='') ? 'AND `language` = \''.$sLangKey.'\' ' : ' ')
              . ''
              . 'ORDER BY `language`, `position` ';
        if (($oResult = $this->oDb->query($sqlSet))) {
            while ( $aPage = $oResult->fetchRow(MYSQLI_ASSOC)) {
                if( !$this->oApp->isPageVisible($aPage['page_id'])) { continue; }
                $aResult[$aPage['language']] = $aPage;
            }
        }
      return $aResult;
    }

    /**
    *
    * search for pages with given page code and create a DB result object
    * @param integer Pagecode to search for
    * @return object result object or null on error
    */
    private function getPageCodeDbResult( $iPageCode )
    {
        $sql = 'SELECT `language`,'
             .        '`page_id`,`level`,`parent`,`root_parent`,`page_code`,`link`,'
             .        '`visibility`,`viewing_groups`,`viewing_users`,`position`,'
             .        '`menu_title`,`page_title`,`tooltip` '
             .  'FROM `'.$this->oDb->sTablePrefix.'pages` '
             .  'WHERE `page_code` = '.$iPageCode.' '
              .   'AND `visibility` NOT IN(\'none\', \'deleted\') '
//              .   'AND `visibility` NOT IN(\'none\', \'hidden\', \'deleted\') '
             .  'ORDER BY `language`, `position`';
        return $this->oDb->query($sql);
    }

    /**
     *
     * @param integer $parent
     * @return database object with given SQL statement
     */
    private function getPageListDbResult($parent)
    {
        $sql = 'SELECT `language`,`tooltip`,'
             .        '`page_id`,`page_title`,`menu_title`, `page_code`, `parent` '
             . 'FROM `'.$this->oDb->sTablePrefix.'pages` '
             . 'WHERE `parent` = '.$parent. ' '
             . 'ORDER BY `language`, `position`';
        return $this->oDb->query($sql);
    }

    private function getPageCodeValues($iPageCode=0)
    {
        $aRetval = [];
        if( ($oRes = $this->getPageCodeDbResult($iPageCode)) )
        {
            while($aPage = $oRes->fetchRow(MYSQLI_ASSOC))
            {
                if (!$this->oApp->page_is_visible($aPage)) {continue;}
                $aRetval[$aPage['language']] = $aPage;
            }
        }
        return $aRetval;
    }

    private function getPageList($parent, $this_page=0 )
    {
        static $entries = [];
        if( ($oLang = $this->getPageListDbResult($parent)) )
        {
            while($value = $oLang->fetchRow(MYSQLI_ASSOC))
            {
                if (( $value['page_id'] != $this_page ) )
                {
                    $entries [$value['page_id']]['language'] = $value['language'];
                    $entries [$value['page_id']]['menu_title'] = $value['menu_title'];
                    $this->getPageList($value['page_id'], $this_page );
                }
            }
        }
        return $entries;
    }

      public function getPageLangDetails(){
          $aLangData = [];
          $aPossiblePages = [];
// get root pages for all used languages
          $aAllowedRootLanguages = $this->getLanguageInUse();
          if(\sizeof($aAllowedRootLanguages) > 1) {
// get all pages witch the same page_code
              $aPossiblePages = $this->getPageCodeValues( $this->oApp->page['page_code']);
// remove all pages from list with not avaliable languages
              $aLangData = \array_intersect_key($aPossiblePages, $aAllowedRootLanguages);
          }
// add Allowed root pages to possible matches
        return \array_merge($aAllowedRootLanguages,$aLangData);
      }

    private function getLangMenuData()
    {
        $aTplData = [];
        $aAvailablePages = $this->getPageLangDetails();
        if (\sizeof($aAvailablePages) > 1) {
            foreach ( $aAvailablePages as $aPage)
            {
                $aTplData[] = array(
                    'sIconUrl'         => $this->oReg->AppUrl . 'modules/'
                                        . \basename(dirname(__FILE__)) . '/',
                    'bCurrentLanguage' => (($aPage['language'] == $this->oReg->Language) ? true : false),
                    'sTargetPageUrl'   => $this->oReg->AppUrl . $this->oReg->PagesDir
                                        . \trim($aAvailablePages[$aPage['language']]['link'],'/')
                                        . $this->oReg->PageExtension,
                    'sPageTitle'       => $aAvailablePages[$aPage['language']]['page_title'],
                    'sFilename'        => $aAvailablePages[$aPage['language']]['language'],
                    'sImageType'       => $this->sExtension,
                    'sToolTip'         => $aAvailablePages[$aPage['language']]['tooltip']
                );
            }
        }
        return $aTplData;
    }

/********************************************************************************************/
//
/********************************************************************************************/

    private function detectIE()
    {
        return \preg_match('/MSIE (.*?);/', $_SERVER['HTTP_USER_AGENT']);
    }

    public function setExtension($sExtension = 'auto')
    {
        if($sExtension == 'auto' || $sExtension == 'svg') {
            $this->sExtension = ($this->detectIE() == true) ? 'png' : 'svg';
        } else {
            $this->sExtension = $sExtension;
        }
        return;
    }

    public function getLangMenu()
    {
        return trim($this->getLangMenuTwig ());
    }

    public function updateDefaultPagesCode()
    {
        $retVal  = false;
        $aLangs  = $this->getLanguageInUse();
        $entries = $this->getPageList(0);
// fill page_code with page_id for default_language
        foreach($entries as $page_id=>$val)
        {
            if ($val['language'] == $this->oReg->DefaultLanguage) {
                if (($retVal = $this->updatePageCode((int)$page_id, 'pages', (int)$page_id ))==false){ break;}
            }
        }
        return $retVal;
    }

    public function getPagesDetail(){
        return $this->getLanguageInUse();

    }

} // end of class
